import re

import typecheck as tc

############################################################################
# copied from test_typecheck_decorator.py:

class expected:
    def __init__(self, e, msg_regexp=None):
        if isinstance(e, Exception):
            self._type, self._msg = e.__class__, str(e)
        elif isinstance(e, type) and issubclass(e, Exception):
            self._type, self._msg = e, msg_regexp
        else:
            raise Exception("usage: 'with expected(Exception)'")

    def __enter__(self):  # make this a context handler
        try:
            pass
        except:
            pass # this is a Python3 way of saying sys.exc_clear()

    def __exit__(self, exc_type, exc_value, traceback):
        assert exc_type is not None, \
            "expected {0:s} to have been thrown".format(self._type.__name__)
        msg = str(exc_value)
        return (issubclass(exc_type, self._type) and
                (self._msg is None or
                 msg.startswith(self._msg) or  # for instance
                 re.match(self._msg, msg)))    # for class + regexp

############################################################################


def test_named_args():
    @tc.typecheck
    def foo(a: int):
        return a
    @tc.typecheck
    def foo_with_default(a: int=4):
        return a
    with expected(tc.InputParameterError(
            "foo() has got an incompatible value for a: 1")):
        foo(a='1')
    with expected(tc.InputParameterError(
            "foo_with_default() has got an incompatible value for a: [1]")):
        foo_with_default(a=[1])


def test_named_arguments_new():
    @tc.typecheck
    def func(a: int):
        return a

    # test named arguments when value matches expected type
    assert func(5) == 5
    assert func(a=10) == 10

    # test named arguments when value doesn't match expected type
    class SomeClass: pass
    with expected(tc.InputParameterError("func() has got an incompatible value for a: ['1']")):
        func(a=['1'])
    #with expected(tc.InputParameterError("func() has got an incompatible value for a: ['1']")):
    #    func(a=SomeClass())


def test_named_arguments_with_default_new():
    @tc.typecheck
    def func(a: int=5):
        return a

    # test named arguments when value matches expected type
    assert func() == 5
    assert func(10) == 10
    assert func(a=15) == 15

    # test named arguments when value doesn't match expected type
    with expected(tc.InputParameterError("func() has got an incompatible value for a: 1")):
        func(a='1')
